@@ -4,7 +4,7 @@
  *
  *			  enhanced checks for plpgsql functions
  *
- * by Pavel Stehule 2013-2025
+ * by Pavel Stehule 2013-2026
  *
  *-------------------------------------------------------------------------
  *
@@ -42,8 +42,18 @@
 
 #endif
 
-#ifdef PG_MODULE_MAGIC
+
+#ifdef PG_MODULE_MAGIC_EXT
+
+PG_MODULE_MAGIC_EXT(
+  .name = "plpgsql_check",
+  .version = "2.8.7"
+);
+
+#else
+
 PG_MODULE_MAGIC;
+
 #endif
 
 PLpgSQL_plugin **plpgsql_check_plugin_var_ptr;
@@ -173,6 +183,29 @@ plpgsql_check_check_ext_version(Oid fn_oid)
 	}
 }
 
+#if PG_VERSION_NUM >= 190000
+
+	StaticAssertVariableIsOfType(&plpgsql_build_datatype, plpgsql_check__build_datatype_t);
+	StaticAssertVariableIsOfType(&plpgsql_compile, plpgsql_check__compile_t);
+	StaticAssertVariableIsOfType(&plpgsql_parser_setup, plpgsql_check__parser_setup_t);
+	StaticAssertVariableIsOfType(&plpgsql_stmt_typename, plpgsql_check__stmt_typename_t);
+	StaticAssertVariableIsOfType(&plpgsql_exec_get_datum_type, plpgsql_check__exec_get_datum_type_t);
+	StaticAssertVariableIsOfType(&plpgsql_recognize_err_condition, plpgsql_check__recognize_err_condition_t);
+	StaticAssertVariableIsOfType(&plpgsql_ns_lookup, plpgsql_check__ns_lookup_t);
+
+#else
+
+//	AssertVariableIsOfType(&plpgsql_build_datatype, plpgsql_check__build_datatype_t);
+//	AssertVariableIsOfType(&plpgsql_compile, plpgsql_check__compile_t);
+//	AssertVariableIsOfType(&plpgsql_parser_setup, plpgsql_check__parser_setup_t);
+//	AssertVariableIsOfType(&plpgsql_stmt_typename, plpgsql_check__stmt_typename_t);
+//	AssertVariableIsOfType(&plpgsql_exec_get_datum_type, plpgsql_check__exec_get_datum_type_t);
+//	AssertVariableIsOfType(&plpgsql_recognize_err_condition, plpgsql_check__recognize_err_condition_t);
+//	AssertVariableIsOfType(&plpgsql_ns_lookup, plpgsql_check__ns_lookup_t);
+
+#endif
+
+
 /*
  * Module initialization
  *
@@ -191,6 +224,31 @@ _PG_init(void)
 
 	pg_bindtextdomain(TEXTDOMAIN);
 
+#if PG_VERSION_NUM >= 190000
+
+	plpgsql_check__build_datatype_p = (plpgsql_check__build_datatype_t)
+		LOAD_EXTERNAL_FUNCTION("$libdir/plpgsql", "plpgsql_build_datatype");
+
+	plpgsql_check__compile_p = (plpgsql_check__compile_t)
+		LOAD_EXTERNAL_FUNCTION("$libdir/plpgsql", "plpgsql_compile");
+
+	plpgsql_check__parser_setup_p = (plpgsql_check__parser_setup_t)
+		LOAD_EXTERNAL_FUNCTION("$libdir/plpgsql", "plpgsql_parser_setup");
+
+	plpgsql_check__stmt_typename_p = (plpgsql_check__stmt_typename_t)
+		LOAD_EXTERNAL_FUNCTION("$libdir/plpgsql", "plpgsql_stmt_typename");
+
+	plpgsql_check__exec_get_datum_type_p = (plpgsql_check__exec_get_datum_type_t)
+		LOAD_EXTERNAL_FUNCTION("$libdir/plpgsql", "plpgsql_exec_get_datum_type");
+
+	plpgsql_check__recognize_err_condition_p = (plpgsql_check__recognize_err_condition_t)
+		LOAD_EXTERNAL_FUNCTION("$libdir/plpgsql", "plpgsql_recognize_err_condition");
+
+	plpgsql_check__ns_lookup_p = (plpgsql_check__ns_lookup_t)
+		LOAD_EXTERNAL_FUNCTION("$libdir/plpgsql", "plpgsql_ns_lookup");
+
+#else
+
 	AssertVariableIsOfType(&plpgsql_build_datatype, plpgsql_check__build_datatype_t);
 	plpgsql_check__build_datatype_p = (plpgsql_check__build_datatype_t)
 		LOAD_EXTERNAL_FUNCTION("$libdir/plpgsql", "plpgsql_build_datatype");
@@ -219,6 +277,8 @@ _PG_init(void)
 	plpgsql_check__ns_lookup_p = (plpgsql_check__ns_lookup_t)
 		LOAD_EXTERNAL_FUNCTION("$libdir/plpgsql", "plpgsql_ns_lookup");
 
+#endif
+
 	DefineCustomBoolVariable("plpgsql_check.regress_test_mode",
 							 "reduces volatile output",
 							 NULL,
@@ -438,7 +498,10 @@ _PG_init(void)
 		shmem_startup_hook = plpgsql_check_profiler_shmem_startup;
 	}
 
-	plpgsql_check_init_pldbgapi2();
+	plch_init_plugin();
+	//simple_plugin_init();
+
+
 	plpgsql_check_passive_check_init();
 	plpgsql_check_profiler_init();
 	plpgsql_check_tracer_init();
